import { Mind } from '../index';
import { Drag } from './drag';
export declare class Graphic {
    /**
     * 节点相关信息缓存，树形结构拍平
     */
    private cacheMap;
    private options;
    private readonly container;
    private readonly viewport;
    private transform;
    private root?;
    private anchor?;
    private resizeObserver;
    private unbindCache;
    private readonly zoom;
    /**
     * 脑图管理器
     * - `viewport`用作判断可视区域
     * - svg连线将会自动注入到`container`中
     * - transform 相关信息将会自动注入到`container`中
     * @param viewport 视窗
     * @param container 容器
     * @param options 配置参数
     */
    constructor(viewport: HTMLElement, container: HTMLElement, options?: Mind.Options);
    /**
     * 同步 options 中的 zoom extent
     */
    private syncZoomExtentOptions;
    /**
     * 获取当前可见的节点
     */
    private getVisibleNodes;
    /**
     * 获取节点定位锚点（左上角）位置，可在节点绘制的时候确定其位置
     * - 推荐所有节点使用`position:absolute;left:0;top:0;`并且配合`transform`来定位，避免出现绘制异常
     * @param id 节点对应id
     */
    getNodeAnchorCoordinate: (id: string) => Mind.Coordinate | undefined;
    private bindEventListener;
    private onTransform;
    /**
     * - 计算节点可见是否改变，如改变，则通知外部
     * - 重渲染连线，销毁不必要的连线
     */
    private refreshVisible;
    private throttleRefreshVisible;
    /**
     * 视窗尺寸变化
     * @private
     */
    private onViewportResize;
    /**
     * 注销事件绑定
     * - 请在销毁组件之前调用
     */
    unbind: () => void;
    /**
     * 判断节点是否可视
     * @param id 节点对应id
     */
    judgeNodeVisible: (id: string) => boolean;
    /**
     * 设定 `options`
     * - 函数不会自动执行重渲染，如果改变的`options`需要重新计算布局等操作，推荐使用 `setData` 驱动数据重渲染
     * @param options 设定选项
     * @param isMerge 是否与之前的`options`做合并操作
     */
    setOptions: (options?: Mind.Options | undefined, isMerge?: boolean) => void;
    /**
     * 生成拖动控制器
     * - 根节点不可拖拽
     * - 当前暂时只有`Mind.ChildAlignMode.structured`布局算法支持拖拽功能
     * @param drag 拖动节点node对象或id
     * @return
     * - 当root（没有调用`setData`）不存在时，或者`drag`为根节点时，返回`undefined`
     * - 正常情况返回 `Drag` 类对象
     */
    dragControllerBuilder: (drag: Mind.Node | string) => Drag | undefined;
    /**
     * 获取渲染层尺寸
     */
    getLayoutSize: () => Mind.Size | undefined;
    /**
     * 获取`id`对应节点
     * @param id 节点`id`
     */
    getNode: (id: string) => Mind.Node | undefined;
    /**
     * 获取`id`对应节点父级
     * @param id 节点`id`
     */
    getParent: (id: string) => Mind.Node | undefined;
    /**
     * 获取`id`对应节点渲染方位
     * @param id 节点`id`
     */
    getNodeOrientation: (id: string) => Mind.Orientation | undefined;
    /**
     * 主动设置位移缩放
     * - 会与之前的`transform`做深度合并
     * - 请注意：`setTransform` 之后 `onTransformChange` 事件依旧会触发
     * - 此方法不受 `zoomExtent.translate`、`zoomExtent.scale` 限制
     * @param transform 位移缩放数据
     * @param duration 周期，如果配置，则执行变换会附带动画效果
     */
    setTransform: (transform: Partial<Mind.Transform>, duration?: number | undefined) => void;
    /**
     * 设定位移
     * - 此方法受到 `zoomExtent.translate` 限制
     * @param translate 位移差(屏幕尺度)
     * @param duration 周期，如果配置，则执行变换会附带动画效果
     */
    translate: (translate: Mind.Coordinate, duration?: number | undefined) => void;
    /**
     * 设定缩放
     * - 此方法受到 `zoomExtent.translate` 限制
     * - 此方法受到 `zoomExtent.scale` 限制
     * @param scale 缩放比
     * @param point 缩放相对点（如不配置或为`undefined`，则默认相对于`viewport`中心缩放）
     * @param duration 动画周期，如配置，则位移会附带动画效果
     */
    scale: (scale: number, point?: Mind.Coordinate | undefined, duration?: number | undefined) => void;
    /**
     * 将某一个节点中心从某个相对位置做位移（其尺度为屏幕尺度）操作
     * - 此方法不受 `zoomExtent.translate` 限制
     * @param config 配置参数
     * @param config.id 节点id
     * @param config.diff 位移差
     * @param config.relative 相对位置
     * @param duration 动画周期，如配置，则位移会附带动画效果
     */
    nodeTranslateTo: (config: {
        id: string;
        diff: Mind.Coordinate;
        relative: Mind.Relative;
    }, duration?: number | undefined) => void;
    /**
     * 获取位移缩放信息
     */
    getTransform: () => Mind.Transform;
    /**
     * 设置锚点节点
     * @param id 锚定节点id(如不设定，则清空锚点，根节点居中，缩放比归一)
     */
    setAnchor: (id?: string | undefined) => void;
    /**
     * 设置/更新数据，启动重渲染
     * - 在重计算定位时，将保持 `anchor` 对应节点在屏幕上的相对位置不变
     * - 如果 `anchor` 没有设定，或者找不到对应节点，则，根节点居中，缩放比重置为1
     * @param root 根数据
     */
    setData: (root: Mind.Root) => void;
    /**
     * 刷新
     */
    refresh: () => void;
    /**
     * 渲染链接到某个`container`下
     * @param container 容器
     */
    connectTo: (container: HTMLElement) => void;
    /**
     * 同步渲染层尺寸到 container 中
     */
    private syncLayoutSize;
    /**
     * 链接各个节点
     */
    private connect;
    /**
     * 计算脑图布局
     * - 请保证调用之前 data 已经通过 setData 设置完备
     */
    private layout;
    /**
     * 计算各节点可见性，以及链接可见性
     */
    private calcVisible;
}
